/*
  Program: Talk_XL.prg
   Author: Gary White
     Date: July 18, 1998

This program is designed to show examples of using ole 
automation to communicate with MS Excel. The example 
uses a transaction table. Among the fields in the 
transaction table are fields for District and SaleAmount.
What we want to accomplish, is to export the sales totals 
by district to Excel and create a pie chart of the result.

This program will create a table named TRANSACT.DBF 
and generate sample data for the demonstration.

*/

#include xldef.h   // Some Excel constant definitions

local oExcel, oRange, oCell, q, r, i, nRow, cFormula, ;
   cRange, nTop, nLeft

// Create instances of our variables so we can use 
// the assignment (:=) operator from here on out.
store 0 to oExcel, oRange, oCell, q, r, i, nRow, ;
    cFormula, cRange, nTop, nLeft

// create a sample table
if not file("transact.dbf")
   create table transact( ;
      District char(10),;
      SaleAmount numeric( 10, 2))
   use transact excl
   generate 10000
   replace all district with ;
      transform(int(random(0)*10)+1,"@L 999")
   index on district tag district
   use
endif

set procedure to progress.wfm additive

f = new progressForm()
f.mdi = false
f.progress1.visible = false
f.text1.text = "Gathering data..."
f.text1.visible = true
f.open()

// Okay, now the program begins
// First generate a query object that will 
// extract the totals by district.
q := new query()
q.sql := "select District, sum(SaleAmount) as Sales " + ;
   "from transact group by District"
q.active := true

// get a reference to the rowset to shorten addressing
r := q.rowset
// make sure we start at the top of the table
r.first()

f.progress1.rangeMax = r.rowCount() * r.fields.size

// now we start Excel
oExcel := new oleAutoclient("excel.application")

// Uncomment this next line if you want 
// to watch it while it happens 
// oExcel.visible = true

// create a new workbook
oExcel.workbooks.add()

// initialize our row pointer
nRow := 1

// First, let's put in column headings
for i = 1 to r.fields.size
   // select a cell
   oExcel.ActiveSheet.cells( nRow, i ).select()
   with ( oExcel.ActiveCell )
      // set the width to match the field length
      ColumnWidth := r.fields[i].length
      // set the content to the field name
      formula := r.fields[i].fieldName
   endwith
endfor

// Let's add one more column to show the percent 
// for each district
oExcel.ActiveSheet.cells( nRow, i ).select()
oExcel.ActiveCell.formula := "%/Total"

// select the entire top row
oRange := oExcel.ActiveSheet.Range("1:1")
// make it boldface
oRange.font.bold := true
// and align it to center text
// you can get this value from Excel's Object Browser
oRange.horizontalAlignment := xlCenter

f.text1.visible = false
f.progress1.visible = true

// now we'll just loop until we run out of records
do while not r.endOfSet
   // increment our row pointer
   nRow ++
   for i = 1 to r.fields.size
	   f.progress1.value ++
      // get a cell reference
      oCell := oExcel.ActiveSheet.cells( nRow, i )
      // store the field contents to the cell
      do case
         case r.fields[i].type = "CHARACTER"
            // if it is a character field, we'll prepend a 
            // single quote to insure that Excel treats it 
            // as text
            oCell.formula := "'"+trim(r.fields[i].value)
            oCell.horizontalAlignment := xlCenter
         case r.fields[i].type = "DOUBLE"
            // otherwise just stuff the value
            oCell.formula := r.fields[i].value
            // Here, I'll cheat a little. The calculated field 
            // is typed as a double. 
            // I want to format that as currency.
            oCell.NumberFormat := "$#,##0.00"
         otherwise
            oCell.formula := r.fields[i].value
      endcase

      // we want to color every other row to dress the 
      // worksheet up and make it easier to read
      if nRow % 2 # 0   // if the row number is odd...
         // select the first three columns of the row
         cRange := "A" + ltrim( str( nRow ) ) + ":C" + ;
            ltrim( str( nRow ) )
         oExcel.Range( cRange ).select()
         // set the fill color
         oExcel.Selection.Interior.ColorIndex := 40
      endif
   endfor
   // get the next record
   r.next()
enddo

f.text = "Starting Excel..."
f.progress1.release()
f.text1.text = "Formatting..."
f.text1.visible = true

// skip down one more row to insert totals
nRow ++

// select the first column
oCell := oExcel.ActiveSheet.cells( nRow, 1 )
// and stuff a label there
with ( oCell )
   // insert the label for the totals
   formula := "All Districts"
   // make it bold
   font.bold := true
endwith

// now we'll create the formula to total 
// all the transactions
cRange := "(B2:B"+ltrim(str(nRow-1))+")"

// our total is in the second column
oCell := oExcel.ActiveSheet.cells( nRow, 2 )
with ( oCell )
   // assign the formula
   formula := "=sum" + cRange
   // format it as currency
   NumberFormat := "$#,##0.00"
   // make it bold
   font.bold := true
endwith

// let's also calculate an average while we're at it
// select the cell in the first column
oCell := oExcel.ActiveSheet.cells( nRow + 1, 1 )
// and stuff a label there
with ( oCell )
   // insert the label for the totals
   formula := "Average"
   // make it bold
   font.bold := true
   // and green
   Font.ColorIndex := 10
endwith

oCell := oExcel.ActiveSheet.cells( nRow + 1, 2)
with ( oCell )
   // assign the formula
   formula := "=average" + cRange
   // format it as currency
   NumberFormat := "$#,##0.00"
   // make it bold
   font.bold := true
   // and green
   Font.ColorIndex := 10
endwith

// select the data portion of the percentage column
cRange := "C2:C" + ltrim( str( nRow - 1 ) )
oExcel.Range( cRange ).select()

// create a formula to calculate the percentage
// addressing in a formula can be relative or absolute
// in our formula we use a relative address to select the 
// cell to the left of our percentage cell and an absolute 
// to address the total at the bottom of the column.
cFormula := "=RC[-1]/R" + ltrim(str(nRow))+"C2"
// and assign it to the selected cells
oExcel.Selection.FormulaR1C1 := cFormula
// format it as percentage with 1 decimal
oExcel.Selection.NumberFormat := "0.0%"

// create a formula to calculate the total 
// of the percentage column
cFormula := "=sum(C2:C"+ltrim(str(nRow-1))+")"
// get a reference to the cell at the bottom of 
// the percentage column
oCell := oExcel.ActiveSheet.cells( nRow, 3 )
with ( oCell )
   // store the formula
   formula := cFormula
   // format it as percentage w/1 decimal
   NumberFormat := "0.0%"
   // and make it bold
   font.bold := true
endwith

// let's insert a blank line at the top
oExcel.Rows("1:1").Select()
oExcel.Selection.Insert()

// Select the first 3 cells in the top row
oExcel.Range("A1:C1").Select()
// and merge them for our title area
oExcel.Selection.Merge()

with( oExcel.Selection )
// align it to center
   HorizontalAlignment := xlCenter
   VerticalAlignment := xlCenter
endwith

// now select our merged cell
oExcel.Range("A1").Select()

// and put in a page heading
with ( oExcel.ActiveCell )
   Formula := "Sales by District"
   font.name := "Times New Roman"
   font.size := 18
   font.bold := true
   Interior.ColorIndex := 53
   Font.ColorIndex := 2
endwith

// increase the row height of our title row
// this automatically sizes to fit the increased 
// font size, but here we want it a little bigger
oExcel.Rows("1:1").select()
oExcel.Selection.RowHeight := 28

// now, let's turn off those ugly grid lines
oExcel.ActiveWindow.DisplayGridlines = False

// select our data
cRange := "A1:B" + ltrim( str( nRow + 2 ) )
oExcel.Range( cRange ).select()
// and put borders on all cells
with( oExcel.Selection.Borders )
   LineStyle := xlContinuous
   Weight := xlThin
   ColorIndex := xlAutomatic
endwith

f.text1.text = "Creating chart..."

// Now, let's create a chart
// start by selecting the data
cRange := "A3:B" + ltrim( str( nRow ) )
oExcel.Range( cRange ).Select()
//  create the chart
oExcel.Charts.Add()
// tell the chart where to get its data
oRange := oExcel.Sheets("Sheet1").Range( cRange )
oExcel.ActiveChart.SetSourceData( oRange, xlColumns )
// put the chart on the same sheet as the data
oExcel.ActiveChart.Location( xlLocationAsObject , "Sheet1" )

f.text1.text = "Formatting chart..."

with ( oExcel.ActiveChart )
   // This will be a 3d pie chart
   ChartType := xl3DPie
   // Now let's put in a title
   HasTitle := True
   // give it some text
   ChartTitle.Text := "Sales by District"
   // pump up the font a little
   ChartTitle.Font.bold := true
   ChartTitle.Font.size := 24
   ChartTitle.Font.name := "Times New Roman"
   ChartTitle.Font.ColorIndex := 2
   // Now, lets give it a gradient fill
   ChartArea.Fill.OneColorGradient(msoGradientHorizontal,4,0.4)
   ChartArea.Fill.Visible := True
   ChartArea.Fill.ForeColor.SchemeColor := 53
   // Let's make the plot area trasparent
   PlotArea.Interior.ColorIndex := xlNone
   PlotArea.Border.LineStyle := xlNone
   // Make the legend font small enough to fit
   Legend.Font.Size := 8
   // and put a drop shadow on it
   Legend.Shadow := true
   // Default 3d elevation is 15 degrees. That's a 
   // little flat for a pie chart, so we'll tip it 
   // up a bit
   Elevation := 40
endwith

f.text1.text = "Done."

// In positioning the chart, the unit of 
// measurement is points

// Move the chart because the default location will 
// likely cover part of our data
nLeft := 0 - ( oExcel.ActiveSheet.shapes("Chart 1").left ) + 225
oExcel.ActiveSheet.Shapes("Chart 1").IncrementLeft( nLeft )

nTop := 0 - ( oExcel.ActiveSheet.shapes("Chart 1").top )
oExcel.ActiveSheet.Shapes("Chart 1").IncrementTop( nTop )

// now let's make it a bit larger
oExcel.ActiveSheet.Shapes("Chart 1").ScaleWidth( 1.15, 0, 0 )
oExcel.ActiveSheet.Shapes("Chart 1").ScaleHeight( 1.5, 0, 0 )

// put a copy of the chart in the clipboard so 
// you can paste it somewhere if you want
oExcel.ActiveSheet.ChartObjects("Chart 1").copy()

// now we'll just hide the cursor behind the chart
oExcel.Range( "E2" ).select()
f.release()
// finally we'll show Excel
oExcel.visible := true

return

